# Called as CohomologyGenerators(C,n), the objective
# of this function is to lift ALL of the generators
# n-1 times, discovering the generators of degree > 1 
# in the process. In particular, this means that the
# degree 1 generators will be lifted to maps P_n -> P_{n-1}.

InstallMethod(CohomologyGenerators,
  "Computes cohomology generators to degree n",
  [IsCObject,IsPosInt], 
  function(C,n)
    local G,K,R,L,B,H,D,P,d,m,c,j,k,l,V,W,BB,N,h; 
    ProjectiveResolution(C,n);
    G:=C!.G; K:=C!.K; R:=C!.R; L:=C!.L; B:=C!.B;
    if not IsBound(C!.H) then C!.H:=List([1..B[2]],x->[]);fi;
    if not IsBound(C!.D) then C!.D:=List([1..B[2]],x->1);fi;
    H:=C!.H; D:=C!.D;
    P:=[];
    # I seem to be clobbering some function P here, but I don't
    # know what P does anyway, so I guess this should be OK.
    BB:=Basis(FullRowSpace(K,B[2]));
    for j in [1..B[2]] do
      if not IsBound(H[j][1]) then
        H[j][1]:=FirstLift(C,BB[j]);
      fi;
    od;

    for m in [Size(H[1])..n-1] do
      h:=Size(H); 
      # h is the number of generators there were at the 
      # beginning of round m. We need this information later.
      
      for j in [1..h] do
        d:=D[j];
        if not IsBound(R[d+m]) then ProjectiveResolution(C,d+m);fi;
        c:=R[d+m]*LiftHom(L[1],H[j][m]);
        H[j][m+1]:=List(c,x->SolutionMat(C!.d[m],x));
    
        # Digging out the first column of each block column of 
        # the big matrix we just computed ammounts to multiplying 
        # eta_j by a degree m map, producing a degree d+m map. 
        # We stick them all into P[d+m] for now. 
    
        if not IsBound(P[d+m]) then P[d+m]:=[];fi;
        for k in [1..B[m+1]] do
          Append(P[d+m],[ExtractColumn(H[j][m+1],(k-1)*Size(G)+1)]);
        od;
      od;
    
      # At this point, all products of generators
      # of degree < m+1 which produce maps of degree m+1
      # have been computed put in P[m+1].
    
      V:=FullRowSpace(K,B[m+2]);
      W:=Subspace(V,P[m+1]);
      BB:=ExtendToBasis(V,Basis(W));
      N:=List([Dimension(W)+1..Dimension(V)],x->BB[x]);
    
      # Voila! The new generators in degree m+1. It only
      # remains to repeat all of the above computations
      # m times for each of the |N| new generators so that
      # all the generators will have the same number of lifts.
    
      for j in [1..Size(N)] do
        D[h+j]:=m+1;
        H[h+j]:=[FirstLift(C,N[j])];
        ProjectiveResolution(C,2*m+1);
        for k in [1..m] do
          c:=R[m+k+1]*LiftHom(L[1],H[h+j][k]);
          H[h+j][k+1]:=List(c,x->SolutionMat(C!.d[k],x));
          if not IsBound(P[m+k+1]) then P[m+k+1]:=[];fi;
          for l in [1..B[k+1]] do
          Append(P[m+k+1],[ExtractColumn(H[h+j][k+1],(l-1)*Size(G)+1)]);
          od;
        od;
      od;
    od;
    return D;
  end
);
