# Returns column n of M.
InstallGlobalFunction(ExtractColumn,function(M,n)
  return List(M,x->x[n]);
  end
);

# Returns a |v|x|G| matrix with v^T in the first column.
InstallGlobalFunction(FirstLift,function(C,v) 
  local N,j;
  N:=NullMat(Size(v),Size(C!.G),C!.K);
  for j in [1..Size(v)] do N[j][1]:=v[j];od;
  return N;
  end
);

# Returns a nxn block diagonal matrix with M on the diagonal.
InstallGlobalFunction(GxGxG,function(M,n)
  return KroneckerProduct(
    IdentityMat(n,DefaultFieldOfMatrix(M)),M);
  end
);

# Returns a list of size n having M in position d.
InstallGlobalFunction(Kronecker,function(M,n,d)
  local L;
  L:=List([1..n],x->0*M);
  L[d]:=M;
  return L;
  end
);

# The following function is taken from the file utils.gi 
# by Gene Cooperman and Scott Murray. utils.gi appeared in
# an earlier version of the GAP library.
InstallGlobalFunction(ExtendToBasis,function(V,vects)
  local subsp, b, newsubsp;
  vects := ShallowCopy(vects);
  subsp := Subspace(V,vects);
  for b in BasisVectors(Basis(V)) do
    newsubsp := Subspace(V,
      Concatenation(BasisVectors(Basis(subsp)),[b]));
    if newsubsp <> subsp then
    subsp := newsubsp;
    Add(vects,b);
    fi;
  od;
  return Basis(V,vects);
  end
);
