# Given a map f:G->H, produces the induced map H(H)->H(G) to degree n.
InstallGlobalFunction(InducedHomomorphismOnCohomology,function(C,D,f,n)
  local L,M,g,B,F,G,i,j,k,c,I,J;
  
  if not C!.K=D!.K then Error("Fields not the same");fi;
  if not Source(f)=C!.G then Error("Source(f) not the right group");fi;
  if not Range(f)=D!.G then Error("Range(f) not the right group");fi;
  
  CohomologyRing(C,n);
  CohomologyRing(D,n);
  B:=Basis(C!.A);
  L:=ReverseLex(C!.G);
  M:=ReverseLex(D!.G);
  g:=List([1..Size(C!.G)],x->List([1..Size(D!.G)],y->Zero(C!.K)));
  for i in [1..Size(L)] do g[i][Position(M,L[i]^f)]:=One(C!.K);od;
  ConvertToMatrixRep(g);
  F:=[g];
  
  # Lift g to a chain map.
  for j in [1..n-1] do
    c:=C!.d[j]*F[j];
    F[j+1]:=List(c,x->SolutionMat(D!.d[j],x));
  od;
  
  # Remove all rows and columns except the first of every block
  G:=[];
  for i in [1..n] do
  G[i]:=List([1..C!.B[i]],
     j->List([1..D!.B[i]],
     k->F[i][(j-1)*Size(C!.G)+1]
            [(k-1)*Size(D!.G)+1]
     ));
  od;
  
  # Bookeeping. Produce the list J of 
  # images of the map we're constructing.
  I:=[];
  for j in [1..n] do
    for k in [1..D!.B[j]] do
      Append(I,
        [Concatenation(
          ListWithIdenticalEntries(Sum(C!.B{[1..j-1]}),0),
          ExtractColumn(G[j],k),
          ListWithIdenticalEntries(Sum(C!.B{[j+1..n+1]}),0)
        )]
      );
    od;
  od;
  for j in [1..D!.B[n+1]] do
    Append(I,[ListWithIdenticalEntries(Sum(C!.B),0)]);
  od;
  J:=List(I,x->Sum([1..Size(B)],y->B[y]*x[y]));
  
  return AlgebraHomomorphismByImages(D!.A,C!.A,Basis(D!.A),J);
  end
);
