InstallGlobalFunction(LiftChainMap,function(C,L,d,n)
  # Lifts the chain map L of degree d n-1 times
  local j,c;
  for j in [Size(L)..n] do
    c:=C!.R[d+j]*LiftHom(C!.L[1],L[j]);
    Append(L,[List((-1)^d*c,x->SolutionMat(C!.d[j],x))]);
  od;
  end
);

InstallGlobalFunction(_MasseyProduct,function(C,L,d,n)
# Recursively compute u such that 
# du = the lower degree product <L> 
# and the first n-1 lifts of u
local m,c,t,l,r,dl,dr,s,f,u,du;
m:=Size(L);
if m=1 then return L[1];fi;

if m>1 then
c:=List([1..n-1],x->0);
for t in [1..m-1] do
  #<1..t>*<t+1..m>
  dl:=Sum(d{[1..t]})-t;       # Degree(l)
  dr:=Sum(d{[t+1..m]})+t-m+1; # Degree(r)
  l:=_MasseyProduct(C,L{[1..t]},d{[1..t]},n+dr-1); # Product on left
  r:=_MasseyProduct(C,L{[t+1..m]},d{[t+1..m]},n-1);# Product on right
  for s in [1..n-1] do
    c[s]:=c[s]+(-1)^dl*l[dr+s+1]*LiftHom(C!.L[1],r[s+1]);
  od;
od;

f:=_MasseyProduct(C,L{[1..m-1]},d{[1..m-1]},d[m])[d[m]+1]
  *LiftHom(C!.L[1],L[m][1]);
u:=[0,List(f,x->SolutionMat(C!.d[1],x))];
if not ForAll(u[2],x->not x=fail) then return fail;fi;
du:=Sum(d)-m+1; # Degree(u)
for t in [1..n-1] do
  f:=c[t]-(-1)^du*C!.R[du+t+1]*LiftHom(C!.L[1],u[t+1]);
  Append(u,[List(f,x->SolutionMat(C!.d[t+1],x))]);
  if not ForAll(u[2+t],x->not x=fail) then return fail;fi;
od;
return u;
fi;
end
);

InstallGlobalFunction(MasseyProduct,function(arg)
local C,m,d,B,H,c,t,s,xy,x,dt;
m:=Size(arg);
C:=FamilyObj(arg[1])!.fullSCAlgebra!.AssociatedCohomologyObject;
d:=List(arg,Degree);
ProjectiveResolution(C,Sum(d)+1-m);
CohomologyRing(C,Sum(d)+1-m);
if not IsBound(C!.P) or Size(C!.P)<Sum(C!.B{[1..m+1]})
  then C!.P:=Concatenation([0],Concatenation(C!.h));fi;
B:=Basis(C!.A);
H:=List(arg,x->Position(B,x));
# Lift all the cocycles in arg a bunch of times
for t in [1..m-1] do
  LiftChainMap(C,C!.P[H[t]],d[t],Sum(d{[t+1..m]}));
od;
xy:=_MasseyProduct(C,
  List([1..m-1],x->C!.P[H[x]]),
  d{[1..m-1]},d[m]
);
if xy=fail then return fail;fi;
x:=(-1)^(Sum(d{[1..m-1]})-m+3)
  *xy[d[m]+1]*LiftHom(C!.L[1],C!.P[H[m]][1]);
dt:=Sum(d)-m+2;
return Coefficients(Basis(FullRowSpace(C!.K,C!.B[dt+1])),
  TransposedMat(x)[1])*
    B{[ Sum(C!.B{[1..dt]})+1..Sum(C!.B{[1..dt+1]}) ]};
end
);
