DeclareRepresentation("IsCObjectCompRep",
  IsComponentObjectRep, ["G","K","M","e","d","L","R","B","H"]);
CObject:=NewType(NewFamily("Oh, this is silly!"),
  IsCObject and IsCObjectCompRep);

InstallMethod(CohomologyObject,
  "Creates a cohomology object",[IsGroup,IsField,IsObject],
  function(G,K,M)
    return Objectify(CObject,
      Immutable(rec(G:=G,K:=K,M:=M)));
  end
);

# We are SOOO flexible; we can EVEN handle 
# the case when K isn't a field!
InstallOtherMethod(CohomologyObject,
  "Creates a cohomology object",[IsGroup,IsRing],
  function(G,K)
    return Objectify(CObject,
      Immutable(rec(G:=G,K:=K,M:=rec(
        field:=K,#isMTXModule:=false,
        dimension:=1,
        generators:=List(Pcgs(G),x->[[One(K)]])
      ))));
  end
);

InstallOtherMethod(CohomologyObject,
  "Creates a cohomology object",[IsGroup],
  function(G)
    local K;
    K:=GF(PrimePGroup(G));
    return CohomologyObject(G,K,
      GModuleByMats(List(Pcgs(G),x->[[One(K)]]),K)
    );
  end
);

# Called as ProjectiveResolution(C,n), the following function computes
# P_n -> P_{n-1} -> ... -> P_1 -> P_0 -> M -> 0.

InstallMethod(ProjectiveResolution,"Computes a minimal projective resolution",
  [IsCObject,IsPosInt], 
  function(C,n)
    local d;
    if not IsBound(C!.e) then C!.e:=FirstStep(C!.M);fi;
    if not IsBound(C!.B) then C!.B:=[];fi;
    if not IsBound(C!.B[1]) then C!.B[1]:=Size(C!.e)/Size(C!.G);fi;
    if not IsBound(C!.L) then C!.L:=PCGens(C!.G,C!.K);fi;
    if not IsBound(C!.R) then C!.R:=[];fi;
    if not IsBound(C!.d) then C!.d:=[];fi;
    if not IsBound(C!.R[1]) then
      C!.R[1]:=GeneratorsOfRadical(C!.e,C!.L[2]);
      C!.d[1]:=LiftHom(C!.L[1],C!.R[1]);
      C!.B[2]:=Size(C!.R[1]);
    fi;
    for d in [Size(C!.R)+1..n] do
      C!.R[d]:=GeneratorsOfRadical(C!.d[d-1],C!.L[2]);
      C!.d[d]:=LiftHom(C!.L[1],C!.R[d]);
      C!.B[d+1]:=Size(C!.R[d]);
    od;
    return C!.B;
  end
);
